function [LogMargChib,LogMargModHarm,Pires,Phires,Sigmares]=marglikedetmodel(X,D,K,niter,difference,exogenous,lambdas,muphi,Sigmaphi,firstlagmean,domesticdummies,MargLikeMethod)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	Computing the marginal likelihood of the determinstic trend
%           adjusted VAR model using Chib's (JASA, 1995) Gibbs sampling
%           approach.
%
% MODEL:    x_{t}-PHI*d_{t}=PI_{1}*(x_{t-1}-PHI*d_{t})+...+PI_{K}*(x_{t-K}-PHI*d_{t})+eps_{t}
%									
% INPUT:	X: 		    (T-by-p) matrix with T observations on p modelled variables.
%			D: 		    (T-by-q) matrix with T observations on q
%			             non-modelled variables. The program adds a column of ones for the constant in the process
%			k:          (scalar) number of lags in the VAR.
%           niter:      (scalar) number of post-burnin Gibbs iterations
%           exogenous   (p2-vector) indicies for the columns in X
%                       (variables) which are exogenous with respect to the other
%                       variables in the model.
%           lambdas     (4-vector) Hyperparameters in prior
%                       Smaller values produces priors which are more concentrated around zero.
%                           - lambdas(1): Baseline tightness parameter.         Limits=(0,inf)  Default value = 0.3
%                           - lambdas(2): Own/others shrinkage                  Limits=(0,1)    Default value = 1
%                           - lambdas(3): Lag length shrinkage parameter.       Limits=(0,inf)  Default value = 1
%                           - lambdas(4): Exogenity parameter                   Limits=(0,1)    Default value = 10^-16  
%           muphi       (pq-by-1) vector of prior means for PHI
%           Sigmaphi    (pq-by-1) vector of prior standard deviations for PHI
%           firstlagmean (p-by-1) vector of prior means for the first lag
%                           coefficient of each time series in the system.
%           domesticdummies (q-by-1) vector Indicates which dummies are purely domestic.
%           MargLikeMethod      MargLikeMethod=1, Chib's marginal likelihood identity
%                       MargLikeMethod=2, Geweke's Modified harmonic estimator
%                       MargLikeMethod=3, Both Chib and ModHarm
%
% OUTPUT:	
%           logmarglike (scalar) log of marginal likelihood of the model
%           Pires       (niter-by-(p^2)*k) matrix of draws from the posterior of
%                       VAR coefficients.
%           Phires      (niter-by-p*q) matrix of draws from the posterior
%                       of the non-modelled variables.
%           Sigmares    (niter-by-p*(p-1)/2) matrix of draws from the
%                       posterior of the covariance matrix of disturbances.
%
% AUTHOR:	Mattias Villani, Research Department, Sveriges Riksbank and Department of Statistics, Stockholm University.
%		    E-mail: mattias.villani@riksbank.se.
%
% DATE:		2003-11-24
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%% Initial stuff %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

muphi=muphi(:);
Sigmaphi=Sigmaphi(:);

LogMargChib=nan;
LogMargModHarm=nan;

% Set seed by clock.
rand('state',sum(100*clock))
randn('state',sum(100*clock))

if MargLikeMethod==2 | MargLikeMethod==3 % ModHarm is to be computed
    ComputeLogPost=1;
else % Only Chib, no need to evaluate LogPost for every draw.
    ComputeLogPost=0;
end

[T,p]=size(X);
Dstar=D;
% Construct Small open economy modified Minnesota prior
for i=1:p
    scales(i)=univariatevar(X(:,i),K,Dstar);
end
[mu_pi,Omega_pi]=build_soe_minn_meancorr(p,exogenous,K,scales,lambdas,difference,firstlagmean,domesticdummies); % mu and Omega is for vec(B).

% Prior on Phi
mu_phi=muphi;
Omega_phi=diag(Sigmaphi.^2);
        
        
% Perform full Gibbs sampling from posterior of Pi, Phi and Sigma
% and compute Pi_{0}, Phi_{0} and Sigma_{0}
D=D(:,1:end-1); % Removing constant as it is added back in varpostmeancorr
[Pires,Phires,Sigmares,LogPost]=varpostmeancorr(X,D,K,niter,difference,exogenous,lambdas,muphi,Sigmaphi,firstlagmean,domesticdummies,ComputeLogPost);

if MargLikeMethod==2 % Only ModHarm
    LogMargLikes = MargDensModHarmonic([Pires Sigmares Phires],LogPost,0.3:0.1:0.7,[],[],1);
    LogMargModHarm=mean(LogMargLikes(:,2));
    return;
end

if MargLikeMethod==3 % Compute both Chib and ModHarm
    LogMargLikes = MargDensModHarmonic([Pires Sigmares Phires],LogPost,0.3:0.1:0.7,[],[],1);
    LogMargModHarm=mean(LogMargLikes(:,2));
end
    

vecPi0=median(Pires)'; % Pi0=[Pi_{1},...,Pi_{K}]'
Pi0=invvec(vecPi0,p*K);

vecPhi0=median(Phires)';
Phi0=invvec(vecPhi0,p);
Sigma0=invvech(median(Sigmares)',p);



% Compute log prior of Phi in the point Phi_{0} (no need to compute prior of Sigma as it is the same across models.
logpriorphi=lograndmndens(vecPhi0,mu_phi,Omega_phi);
logpriorpi=lograndmndens(vecPi0,mu_pi,Omega_pi);



% Compute log likelihood in the point (Pi,Phi,Sigma)=(Pi_{0},Phi_{0},Sigma_{0})
[Yphi,Xphi]=datmanip_cond_on_phi(X,Dstar,Phi0,K);
loglik=-(T/2)*log(det(2*pi*Sigma0))-0.5*trace(inv(Sigma0)*(Yphi-Xphi*Pi0)'*(Yphi-Xphi*Pi0));





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       
%%%  Evaluate p(Pi_{0}|X,D_{j}) by Gibbs average  %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

InvOmega_pi=inv(Omega_pi);
InvOmega_pi_mu=InvOmega_pi*mu_pi;
logpi0dens=zeros(niter,1);

for i=1:niter
    Phi=invvec(Phires(i,:)',p);
    Sigma=invvech(Sigmares(i,:)',p);
    [Yphi,Xphi]=datmanip_cond_on_phi(X,Dstar,Phi,K);
    Omega_pi_bar=inv(kron(inv(Sigma),Xphi'*Xphi)+InvOmega_pi);
    mu_pi_bar=Omega_pi_bar*(vec(Xphi'*Yphi*inv(Sigma))+InvOmega_pi_mu);
    logpi0dens(i)=lograndmndens(vecPi0,mu_pi_bar,Omega_pi_bar);
end
c1=mean(logpi0dens);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  Evaluate p(Sigma_{0}|Pi_{0},X,D_{j}) by reduced Gibbs average  %%%
%%%%%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Perform reduced Gibbs sampler with Pi fixed at Pi0.
[Phiresred,Sigmaresred]=varpostmeancorrred(X,D,K,niter,difference,exogenous,lambdas,muphi,Sigmaphi,firstlagmean,domesticdummies,Pi0);
        
% Evaluate p(Sigma_{0}|Pi_{0},X,D_{j}) by Gibbs average of
% p(Sigma_{0}|Pi_{0},Phi,X,D_{j}) over the Reduced Gibbs draws of Phi.

logsigma0dens=zeros(niter,1);
for i=1:niter
    Phi=invvec(Phiresred(i,:)',p);
    Sigma=invvech(Sigmaresred(i,:)',p);
    [Yphi,Xphi]=datmanip_cond_on_phi(X,Dstar,Phi,K);
    logsigma0dens(i)=lograndiwdens(Sigma,(Yphi-Xphi*Pi0)'*(Yphi-Xphi*Pi0),T);
end
c2=mean(logsigma0dens);





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  Eval p(Phi_{0}|Pi_{0},Sigma_{0},X,D_{j}) by formula  %%%
%%%%%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

q=size(Dstar,2);
[Yma,Xma,Dma]=datmanip_meanadj(X,Dstar,K);
DD=Dma'*Dma;
InvOmega_phi=inv(Omega_phi);
invOmega_phi_mu=InvOmega_phi*mu_phi;
U=[eye(p*q) zeros(p*q,p*q*K)]';
Pi0prime=Pi0';
for j=1:K
    U(1+j*p*q:(j+1)*p*q,:)=kron(eye(q),Pi0prime(:,1+(j-1)*p:j*p));
end
Omega_phi_bar=inv(U'*kron(DD,inv(Sigma0))*U+InvOmega_phi);
mu_phi_bar=Omega_phi_bar*(U'*vec(inv(Sigma0)*(Yma-Xma*Pi0)'*Dma)+invOmega_phi_mu);    
c3=lograndmndens(vecPhi0,mu_phi_bar,Omega_phi_bar);


% Evaluates marginal likelihood by Chib's Basic marginal likelihood
% identity
%[loglik logpriorpi logpriorphi -c1 -c2 -c3]
LogMargChib=loglik+logpriorpi+logpriorphi-(c1+c2+c3);

  