function [Phires,Sigmares]=varpostmeancorrred(X,D,K,niter,difference,exogenous,lambdas,muphi,Sigmaphi,firstlagmean,domesticdummies,Pi)
                                                                                                                                                            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	Reduced Gibbs sampler conditional on Pi=[Pi_{1},..,Pi_{K}]'.
%           Determinstic trend adjusted VAR model
%
% MODEL:    x_{t}-PHI*d_{t}=PI_{1}*(x_{t-1}-PHI*d_{t})+...+PI_{K}*(x_{t-K}-PHI*d_{t})+eps_{t}
%									
% INPUT:	X: 		    (T-by-p) matrix with T observations on p modelled variables.
%			D: 		    (T-by-q) matrix with T observations on q
%			             non-modelled variables. The program adds a column of ones for the constant in the process
%			k:          (scalar) number of lags in the VAR.
%           niter:      (scalar) number of post-burnin Gibbs iterations
%           exogenous   (p2-vector) indicies for the columns in X
%                       (variables) which are exogenous with respect to the other
%                       variables in the model.
%           lambdas     (4-vector) Hyperparameters in prior
%                       Smaller values produces priors which are more concentrated around zero.
%                           - lambdas(1): Baseline tightness parameter.         Limits=(0,inf)  Default value = 0.3
%                           - lambdas(2): Own/others shrinkage                  Limits=(0,1)    Default value = 1
%                           - lambdas(3): Lag length shrinkage parameter.       Limits=(0,inf)  Default value = 1
%                           - lambdas(4): Exogenity parameter                   Limits=(0,1)    Default value = 10^-16  
%           muphi       (pq-by-1) vector of prior means for PHI
%           Sigmaphi    (pq-by-1) vector of prior standard deviations for PHI
%           
% OUTPUT:	Pires       (niter-by-(p^2)*k) matrix of draws from the posterior of
%                       VAR coefficients.
%           Phires      (niter-by-p*q) matrix of draws from the posterior
%                       of the non-modelled variables.
%           Sigmares    (niter-by-p*(p-1)/2) matrix of draws from the
%                       posterior of the covariance matrix of disturbances.
%
% AUTHOR:	Mattias Villani, Research Department, Sveriges Riksbank and Department of Statistics, Stockholm University.
%		    E-mail: mattias.villani@riksbank.se.
%
% DATE:		2003-11-12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

muphi=muphi(:);
Sigmaphi=Sigmaphi(:);

% Set seed by clock.
rand('state',sum(100*clock))
randn('state',sum(100*clock))

% Adding a constant to D
[T,p]=size(X);
D=[D ones(T,1)];
q=size(D,2);

% ML estimates as initial values in Gibbs sampling
[Yphi,Xphi]=datmanip_cond_on_phi(X,D,invvec(muphi,p),K);
[Pi1,Phi1,Sigma1]=mlvarmeanadj(Yphi,[],K);
[Piml,Phi,Sigma]=mlvarmeanadj(X,D,K);




% Construct Small open economy modified Minnesota prior (only for Pi)
for i=1:p
    scales(i)=univariatevar(X(:,i),K,D);
end
[mu_pi,Omega_pi]=build_soe_minn_meancorr(p,exogenous,K,scales,lambdas,difference,firstlagmean,domesticdummies); % mu and Omega is for vec(B).

% Prior on Phi
mu_phi=muphi;
Omega_phi=diag(Sigmaphi.^2);

% Computes datamatrices to be used in the full conditional posterior of Phi
[Yma,Xma,Dma]=datmanip_meanadj(X,D,K);
[T,p]=size(Yma);

% Direct posterior simulation p(PI,Sigma)=p(PI|Sigma)*p(Sigma)
Sigmares=zeros(niter,p*(p+1)/2);
Phires=zeros(niter,p*q);
DD=Dma'*Dma;
InvOmega_pi=inv(Omega_pi);
InvOmega_pi_mu=InvOmega_pi*mu_pi;
InvOmega_phi=inv(Omega_phi);
invOmega_phi_mu=InvOmega_phi*mu_phi;
U=[eye(p*q) zeros(p*q,p*q*K)]';
Piprime=Pi';

for i=1:niter
    if rem(i,1000)==0
        disp(i)
    end
    [Yphi,Xphi]=datmanip_cond_on_phi(X,D,Phi,K);
    % Sigma step
    Sigma=randiw((Yphi-Xphi*Pi)'*(Yphi-Xphi*Pi),T);
    invSigma=inv(Sigma);
    Sigmares(i,:)=vech(Sigma)';
   
    % Phi-step
    for j=1:K
        U(1+j*p*q:(j+1)*p*q,:)=kron(eye(q),Piprime(:,1+(j-1)*p:j*p));
    end
    Omega_phi_bar=inv(U'*kron(DD,invSigma)*U+InvOmega_phi);
    mu_phi_bar=Omega_phi_bar*(U'*vec(invSigma*(Yma-Xma*Pi)'*Dma)+invOmega_phi_mu);
    vecPhi=randmn(mu_phi_bar,Omega_phi_bar); % vecPhi=vec(Phi)
    Phi=invvec(vecPhi,p);
    Phires(i,:)=vecPhi';
end




